package de.mpg.mpimet.yacgui.xml;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

public enum TimestepUnit {
	
	SECOND("second", Type.T_INTEGER),
	MINUTE("minute", Type.T_INTEGER),
	HOUR("hour", Type.T_INTEGER),
	DAY("day", Type.T_INTEGER),
	MONTH("month", Type.T_INTEGER),
	YEAR("year", Type.T_INTEGER),
	MILLISECOND("millisecond", Type.T_INTEGER),
	ISO_FORMAT("ISO_format", Type.T_STRING),
	STRING("string", Type.T_STRING);
	
	private final String name;
	
	/**
	 * Type of unit values.
	 * ISO_FORMAT, STRING are string values, others are integers
	 */
	private final int unitType;
	
	private static class Type {
        public static final int T_INTEGER = 1;
        public static final int T_STRING = 2;
    }
	
	private TimestepUnit(String s, int t) {
        name = s;
        unitType = t;
    }
	
	public static TimestepUnit fromString(String name) {
		if (name == null) return null;
		if (name.equals(SECOND.name)) return SECOND;
		else if (name.equals(MINUTE.name)) return MINUTE;
		else if (name.equals(HOUR.name)) return HOUR;
		else if (name.equals(DAY.name)) return DAY;
		else if (name.equals(MONTH.name)) return MONTH;
		else if (name.equals(YEAR.name)) return YEAR;
		else if (name.equals(MILLISECOND.name)) return MILLISECOND;
		else if (name.equals(ISO_FORMAT.name)) return ISO_FORMAT;
		else if (name.equals(STRING.name)) return STRING;
		return null;
	}
	
	/**
	 * @return String representation of TimestepUnit, e.g. "second".
	 */
	@Override
	public String toString() {
		return name;
	}

	/**
	 * Checks if current type is compatible with provided.
	 * Here compatibility means that values of current type can have another type.
	 * Example: integer values can be formatted as strings, but not vice versa.
	 * 
	 * @param toCompare
	 * @return true if time units are identical
	 */
	public boolean isTypeCompatibleWith(TimestepUnit toCompare) {
		return toCompare != null && 
				(unitType == Type.T_INTEGER && toCompare.unitType == Type.T_INTEGER ||
				unitType == Type.T_INTEGER && toCompare.unitType == Type.T_STRING ||
				unitType == Type.T_STRING && toCompare.unitType == Type.T_STRING);
	}
}
