/**
 * @file interp_grid.h
 *
 * @copyright Copyright  (C)  2019 Moritz Hanke <hanke@dkrz.de>
 *                                 Thomas Jahns <jahns@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Thomas Jahns <jahns@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INTERP_GRID_H
#define INTERP_GRID_H

#include "dist_grid.h"

struct interp_field {
  enum yac_location location;
  coordinate_pointer coordinates;
  int * mask;
};

void yac_interp_field_free(struct interp_field interp_field);

struct interp_grid * yac_interp_grid_new(
  struct dist_grid_pair * grid_pair, int src_grid_id, int tgt_grid_id,
  size_t num_src_fields, struct interp_field const * src_fields,
  struct interp_field const tgt_field);

void yac_interp_grid_get_src_cell(
  struct interp_grid * interp_grid, size_t src_cell_idx,
  struct grid_cell * cell);

void yac_interp_grid_get_src_points(
  struct interp_grid * interp_grid, size_t src_field_idx,
  size_t ** src_indices, size_t * count);

void yac_interp_grid_get_tgt_points(
  struct interp_grid * interp_grid, size_t ** tgt_indices, size_t * count);

struct remote_point * yac_interp_grid_get_src_remote_points(
  struct interp_grid * interp_grid, size_t src_field_idx,
  size_t * src_points, size_t count);

void yac_intep_grid_src_global_to_local(
  struct interp_grid * interp_grid, size_t src_field_idx,
  yac_int * src_global_ids, size_t count, size_t * src_local_ids);

struct remote_point * yac_interp_grid_get_tgt_remote_points(
  struct interp_grid * interp_grid, size_t * tgt_points, size_t count);

enum yac_location yac_interp_grid_get_tgt_field_location(
  struct interp_grid * interp_grid);

void yac_interp_grid_get_src_global_ids(
  struct interp_grid * interp_grid, size_t * src_points, size_t count,
  size_t src_field_idx, yac_int * src_global_ids);

void yac_interp_grid_get_tgt_global_ids(
  struct interp_grid * interp_grid, size_t * tgt_points, size_t count,
  yac_int * tgt_global_ids);

void yac_interp_grid_get_src_coordinates(
  struct interp_grid * interp_grid, size_t * src_points, size_t count,
  size_t src_field_idx, coordinate_pointer src_coordinates);

void yac_interp_grid_get_tgt_coordinates(
  struct interp_grid * interp_grid, size_t * tgt_points, size_t count,
  coordinate_pointer tgt_coordinates);

size_t yac_interp_grid_get_num_src_fields(struct interp_grid * interp_grid);

enum yac_location yac_interp_grid_get_src_field_location(
  struct interp_grid * interp_grid, size_t src_field_idx);

const_yac_int_pointer yac_interp_grid_get_src_field_global_ids(
  struct interp_grid * interp_grid, size_t src_field_idx);

const_coordinate_pointer yac_interp_grid_get_src_field_coords(
  struct interp_grid * interp_grid, size_t src_field_idx);

const_coordinate_pointer yac_interp_grid_get_tgt_field_coords(
  struct interp_grid * interp_grid);

const_int_pointer yac_interp_grid_get_src_field_mask(
  struct interp_grid * interp_grid, size_t src_field_idx);

void yac_interp_grid_do_points_search(
  struct interp_grid * interp_grid, coordinate_pointer search_coords,
  size_t count, size_t * src_cells);

void yac_interp_grid_do_points_search_gc(
  struct interp_grid * interp_grid, coordinate_pointer search_coords,
  size_t count, size_t * src_cells);

void yac_interp_grid_do_nnn_search_src(
  struct interp_grid * interp_grid, coordinate_pointer search_coords,
  size_t count, size_t n, size_t * src_points);

void yac_interp_grid_do_nnn_search_tgt(
  struct interp_grid * interp_grid, coordinate_pointer search_coords,
  size_t count, size_t n, size_t * tgt_points);

void yac_interp_grid_do_bnd_circle_search_src(
  struct interp_grid * interp_grid, const_bounding_circle_pointer bnd_circles,
  size_t count, size_t ** src_cells, size_t * num_src_per_tgt);

void yac_interp_grid_do_bnd_circle_search_tgt(
  struct interp_grid * interp_grid, const_bounding_circle_pointer bnd_circles,
  size_t count, size_t ** tgt_cells, size_t * num_tgt_per_src);

struct remote_point_infos * yac_interp_grid_determine_remote_point_infos_src(
  struct interp_grid * interp_grid, size_t src_field_idx,
  size_t count, yac_int * src_global_ids);

struct remote_point_infos * yac_interp_grid_determine_remote_point_infos_tgt(
  struct interp_grid * interp_grid, size_t count, yac_int * tgt_global_ids);

void yac_interp_grid_determine_local_pos_src(
    struct interp_grid * interp_grid, size_t src_field_idx,
    yac_int * src_global_ids, size_t count, size_t * src_idx);

Xt_redist yac_interp_grid_generate_redist_src_get(
  struct interp_grid * interp_grid, size_t src_field_idx,
  yac_int * src_global_ids, size_t count);

Xt_redist yac_interp_grid_generate_redist_tgt_put(
  struct interp_grid * interp_grid,
  struct remote_point_infos * tgt_infos, size_t count);

MPI_Comm yac_interp_grid_get_MPI_Comm(struct interp_grid * interp_grid);

struct const_basic_grid_data * yac_interp_grid_get_basic_grid_data_src(
  struct interp_grid * interp_grid);

struct const_basic_grid_data * yac_interp_grid_get_basic_grid_data_tgt(
  struct interp_grid * interp_grid);

/**
 * the neighbours returned by this routine are sorted per cell in clockwise or
 * counter-clockwise order
 */
void yac_interp_grid_get_src_cell_neighbours(
  struct interp_grid * interp_grid, size_t * src_cells, size_t count,
  size_t * neighbours);

/**
 * Generates for each requested vertex, the list of cell surrounding it will.\n
 * If the vertex is not completely surrounded by cells, num_cells_per_vertex
 * will be zero for this vertex.\n
 * The cells are listed such that neighbouring cells are next to each other or
 * at the end and beginning of the list.\n
 * The lists are decomposition independant.
 */
void yac_interp_grid_get_aux_grid_cells_src(
  struct interp_grid * interp_grid, size_t * vertices, size_t count,
  size_t ** cells, int * num_cells_per_vertex);

/**
 * Generates lists for surrounding cell for all vertices associated to the
 * provided list of cells.
 * If the vertex is not completely surrounded by cells, num_cells_per_vertex
 * will be zero for this vertex.\n
 * The cells are listed such that neighbouring cells are next to each other or
 * at the end and beginning of the list.\n
 * The lists are decomposition independant.
 */
void yac_interp_grid_get_aux_grid_src(
  struct interp_grid * interp_grid, size_t * cells, size_t count,
  size_t ** vertex_to_cell, size_t ** vertex_to_cell_offsets,
  int ** num_cells_per_vertex);

/**
 * Relocates source-target-point-pairs. The flag "to_tgt_owner" determines
 * whether the pairs go to the dist owner process of the target or source
 * points for each pair.
 */
void yac_interp_grid_relocate_src_tgt_pairs(
  struct interp_grid * interp_grid, int to_tgt_owner,
  size_t src_field_idx, size_t ** src_points, size_t ** tgt_points,
  size_t * count);

void yac_interp_grid_delete(struct interp_grid * interp_grid);

#endif // INTERP_GRID_H
