/**
 * @file quicksort.c
 *
 * @copyright Copyright  (C)  2019 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include "utils.h"

#define YAC_QSORT_ARRAY_TYPE int
#define YAC_QSORT_INDEX_TYPE int
#define YAC_QSORT_NAME yac_quicksort_index
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE yac_int
#define YAC_QSORT_INDEX_TYPE size_t
#define YAC_QSORT_NAME yac_quicksort_index_yac_int_size_t
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE yac_int
#define YAC_QSORT_INDEX_TYPE uint64_t
#define YAC_QSORT_NAME yac_quicksort_index_yac_int_uint64_t
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE yac_int
#define YAC_QSORT_INDEX_TYPE int
#define YAC_QSORT_NAME yac_quicksort_index_yac_int_int
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE size_t
#define YAC_QSORT_INDEX_TYPE size_t
#define YAC_QSORT_NAME yac_quicksort_index_size_t_size_t
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE uint64_t
#define YAC_QSORT_INDEX_TYPE size_t
#define YAC_QSORT_NAME yac_quicksort_index_uint64_t_size_t
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE int
#define YAC_QSORT_INDEX_TYPE size_t
#define YAC_QSORT_NAME yac_quicksort_index_int_size_t
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE size_t
#define YAC_QSORT_INDEX_TYPE yac_int
#define YAC_QSORT_NAME yac_quicksort_index_size_t_yac_int
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE size_t
#define YAC_QSORT_INDEX_TYPE int
#define YAC_QSORT_NAME yac_quicksort_index_size_t_int
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE size_t
#define YAC_QSORT_INDEX_TYPE void*
#define YAC_QSORT_NAME yac_quicksort_index_size_t_void_p
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE int
#define YAC_QSORT_INDEX_TYPE yac_int
#define YAC_QSORT_NAME yac_quicksort_index_int_yac_int
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE

#define YAC_QSORT_ARRAY_TYPE int
#define YAC_QSORT_INDEX_TYPE double
#define YAC_QSORT_NAME yac_quicksort_index_int_double
#include "quicksort_template.h"
#undef YAC_QSORT_NAME
#undef YAC_QSORT_INDEX_TYPE
#undef YAC_QSORT_ARRAY_TYPE
